//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.AddCustomGetHandler)  return string.Empty; #>
using System;
using System.IO;
using System.Reflection;
//!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
using System.Web;
using System.Web.UI;
//!<# } #>
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Class1;
using ITHit.WebDAV.Server.Extensibility;

namespace ITHit.WebDav.Server.SampleModel
{
    //$<IMethodHandler
    /// <summary>
    /// This handler processes GET and HEAD requests to folders returning custom HTML page.
    /// </summary>
    internal class MyCustomGetHandler : IMethodHandler
    {
        /// <summary>
        /// Handler for GET and HEAD request registered with the engine before registering this one.
        /// We call this default handler to handle GET and HEAD for files, because this handler
        /// only handles GET and HEAD for folders.
        /// </summary>
        public IMethodHandler OriginalHandler { get; set; }

        /// <summary>
        /// Gets a value indicating whether output shall be buffered to calculate content length.
        /// Don't buffer output to calculate content length.
        /// </summary>
        public bool EnableOutputBuffering
        {
            get { return false; }
        }

        /// <summary>
        /// Gets a value indicating whether engine shall log response data (even if debug logging is on).
        /// </summary>
        public bool EnableOutputDebugLogging
        {
            get { return false; }
        }

        /// <summary>
        /// Gets a value indicating whether the engine shall log request data.
        /// </summary>
        public bool EnableInputDebugLogging
        {
            get { return false; }
        }

        /// <summary>
        /// Handles GET request.
        /// </summary>
        /// <param name="context">Instace of <see cref="DavContextBase"/>.</param>
        /// <param name="item">Instance of <see cref="IHierarchyItem"/> which was returned by
        /// <see cref="DavContextBase.GetHierarchyItem"/> for this request.</param>
        public void ProcessRequest(DavContextBase context, IHierarchyItem item)
        {
            //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
            string exePath = Path.GetDirectoryName(new Uri(Assembly.GetExecutingAssembly().CodeBase).LocalPath);
            string htmlPath = Directory.GetParent(exePath).FullName;
            //!<# } #>
            if (item is IFolder)
            {
                // Remember to call EnsureBeforeResponseWasCalled here if your context implementation
                // makes some useful things in BeforeResponse.
                context.EnsureBeforeResponseWasCalled();

                //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
                Page page;
                page = (Page)System.Web.Compilation.BuildManager.CreateInstanceFromVirtualPath(
                    "~/MyCustomHandlerPage.aspx", typeof(MyCustomHandlerPage));
                page.ProcessRequest(HttpContext.Current);
                //!<# } #>

                //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
                using (TextReader reader = new StreamReader(Path.Combine(htmlPath, "MyCustomHandlerPage.html")))
                {
                    string html = reader.ReadToEnd();
                    html = html.Replace("_webDavServerRoot_", context.Request.ApplicationPath.TrimEnd('/'));
                    html = html.Replace("_webDavServerVersion_",
                        Assembly.GetAssembly(typeof(DavEngine)).GetName().Version.ToString());

                    context.Response.ContentLength = html.Length;
                    context.Response.ContentType = "text/html";
                    if (context.Request.HttpMethod == "GET")
                    {
                        using (var writer = new StreamWriter(context.Response.OutputStream))
                        {
                            writer.Write(html);
                        }
                    }
                }
                //!<# } #>
            }
            //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
            else if (context.Request.RawUrl.StartsWith("/AjaxFileBrowser/"))
            {
                context.EnsureBeforeResponseWasCalled();
                using (TextReader reader = new StreamReader(htmlPath + context.Request.RawUrl.Replace("/", "\\")))
                {
                    string html = reader.ReadToEnd();

                    context.Response.ContentLength = html.Length;
                    context.Response.ContentType = "text/html";
                    if (context.Request.HttpMethod == "GET")
                    {
                        using (var writer = new StreamWriter(context.Response.OutputStream))
                        {
                            writer.Write(html);
                        }
                    }
                }
            }
            //!<# } #>
            else
            {
                OriginalHandler.ProcessRequest(context, item);
            }
        }
        
        /// <summary>
        /// This handler shall only be invoked for IFolder items or if original handler (which
        /// this handler substitutes) shall be called for the item.
        /// </summary>
        /// <param name="item">Instance of <see cref="IHierarchyItem"/> which was returned by
        /// <see cref="DavContextBase.GetHierarchyItem"/> for this request.</param>
        /// <returns>Returns <c>true</c> if this handler can handler this item.</returns>
        public bool AppliesTo(IHierarchyItem item)
        {
            return item is IFolder || OriginalHandler.AppliesTo(item);
        }
    }
    //$>
}
