﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if ((Params.Hosting != HostingEnvironment.IIS) || (!Params.CalDav && !Params.CardDav))  return string.Empty; #>
using System;
using System.Configuration;
using System.IO;
using System.Web;
using System.Security.Principal;
using System.Security.AccessControl;

//!<# if (Params.CalDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CardDav;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__
{
    /// <summary>
    /// This module creates initial calendar(s) and addressbook(s) for user during first login.
    /// </summary>
    /// <remarks>
    /// Methods in this module are using impersonation. In case you run IIS Express and log-in 
    /// as the user that is different from the one running IIS Express, the IIS Express must 
    /// run with Administrative permissions.
    /// </remarks>
    public class ProvisioningModule : IHttpModule
    {
        /// <summary>
        /// Path to the folder which stores WebDAV files.
        /// </summary>
        private static readonly string repositoryPath = ConfigurationManager.AppSettings["RepositoryPath"] ?? string.Empty;

        public void Dispose()
        {

        }

        public void Init(HttpApplication application)
        {
            application.PostAuthenticateRequest += App_OnPostAuthenticateRequest;
        }

        private void App_OnPostAuthenticateRequest(object source, EventArgs eventArgs)
        {
            //!<# if (Params.CalDav) { #>
            // Create calendars for the user during first login
            CreateCalendarFolders();
            //!<# } #>
            //!<# if (Params.CardDav) { #>
            // Create addressboks for the user during first login
            CreateAddressbookFolders();
            //!<# } #>
        }
        //!<# if (Params.CalDav) { #>

        /// <summary>
        /// Creates initial calendars for user as well as inbox and outbox folders.
        /// </summary>
        private static void CreateCalendarFolders()
        {
            string physicalRepositoryPath = repositoryPath.StartsWith("~") ?
                HttpContext.Current.Server.MapPath(repositoryPath) : repositoryPath;

            // get path to user folder /calendars/[user_name]/ and check if it exists
            string calendarsUserFolder = string.Format("{0}\\{1}", CalendarsRootFolder.CalendarsRootFolderName, UserName);
            string pathCalendarsUserFolder = Path.Combine(physicalRepositoryPath, calendarsUserFolder);
            if (!Directory.Exists(pathCalendarsUserFolder))
            {
                Directory.CreateDirectory(pathCalendarsUserFolder);

                // grant full control to loged-in user
                GrantFullControl(pathCalendarsUserFolder);

                // create all subfolders under the loged-in user account
                // so all folders has loged-in user as the owner
                DavContext context = new DavContext(HttpContext.Current);
                context.FileOperation(
                    () =>
                    {
                        // make the loged-in user the owner of the new folder
                        MakeOwner(pathCalendarsUserFolder);

                        // create user calendars, such as /calendars/[user_name]/Calendar/
                        string pathCalendar = Path.Combine(pathCalendarsUserFolder, "Calendar");
                        Directory.CreateDirectory(pathCalendar);
                        pathCalendar = Path.Combine(pathCalendarsUserFolder, "Home");
                        Directory.CreateDirectory(pathCalendar);
                        pathCalendar = Path.Combine(pathCalendarsUserFolder, "Work");
                        Directory.CreateDirectory(pathCalendar);

                        // create outbox folder /calendars/[user_name]/outbox/
                        string pathOutbox = Path.Combine(pathCalendarsUserFolder, ScheduleOutboxFolder.ScheduleOutboxFolderName);
                        Directory.CreateDirectory(pathOutbox);

                        // create inbox folder /calendars/[user_name]/inbox/
                        string pathInbox = Path.Combine(pathCalendarsUserFolder, ScheduleInboxFolder.ScheduleInboxFolderName);
                        Directory.CreateDirectory(pathInbox);
                    });
            }
        }
        //!<# } #>
        //!<# if (Params.CardDav) { #>

        /// <summary>
        /// Creates initial addrsessbooks for user.
        /// </summary>
        private static void CreateAddressbookFolders()
        {
            string physicalRepositoryPath = repositoryPath.StartsWith("~") ?
                HttpContext.Current.Server.MapPath(repositoryPath) : repositoryPath;

            // get path to user folder /addrsessbooks/[user_name]/ and check if it exists
            string addressbooksUserFolder = string.Format("{0}\\{1}", AddressbooksRootFolder.AddressbooksRootFolderName, UserName);
            string pathAddressbooksUserFolder = Path.Combine(physicalRepositoryPath, addressbooksUserFolder);
            if (!Directory.Exists(pathAddressbooksUserFolder))
            {
                Directory.CreateDirectory(pathAddressbooksUserFolder);

                // grant full control to loged-in user
                GrantFullControl(pathAddressbooksUserFolder);

                // create all subfolders under the loged-in user account
                // so all folders has loged-in user as the owner
                DavContext context = new DavContext(HttpContext.Current);
                context.FileOperation(
                    () =>
                    {
                        // make the loged-in user the owner of the new folder
                        MakeOwner(pathAddressbooksUserFolder);

                        // create user addressbooks, such as /addressbooks/[user_name]/Addressbook/
                        string pathAddressbook = Path.Combine(pathAddressbooksUserFolder, "Addressbook");
                        Directory.CreateDirectory(pathAddressbook);
                        pathAddressbook = Path.Combine(pathAddressbooksUserFolder, "Business");
                        Directory.CreateDirectory(pathAddressbook);
                    });
            }
        }
        //!<# } #>
        //!<# if (Params.CalDav || Params.CardDav) { #>

        /// <summary>
        /// Gets current authenticated user name. For the sake of simplicity we truncate domain.
        /// </summary>
        private static string UserName
        {
            get
            {
                string name = HttpContext.Current.User.Identity.Name;
                return name.Substring(name.IndexOf("\\", StringComparison.InvariantCulture) + 1);
            }
        }

        /// <summary>
        /// Makes the loged-in user the owner of the folder.
        /// </summary>
        /// <param name="folderPath">folder path in file system</param>
        private static void MakeOwner(string folderPath)
        {
            WindowsIdentity identity = (WindowsIdentity)HttpContext.Current.User.Identity;
            DirectorySecurity securityOwner = Directory.GetAccessControl(folderPath, AccessControlSections.Owner);
            securityOwner.SetOwner(identity.User);
            Directory.SetAccessControl(folderPath, securityOwner);
        }

        /// <summary>
        /// Grants full controll to currently loged-in user.
        /// </summary>
        /// <param name="folderPath">folder path in file system</param>
        private static void GrantFullControl(string folderPath)
        {
            DirectoryInfo folder = new DirectoryInfo(folderPath);
            DirectorySecurity security = folder.GetAccessControl();
            WindowsIdentity identity = (WindowsIdentity)HttpContext.Current.User.Identity;
            security.AddAccessRule(new FileSystemAccessRule(identity.User, FileSystemRights.FullControl, InheritanceFlags.ObjectInherit | InheritanceFlags.ContainerInherit, PropagationFlags.None, AccessControlType.Allow));
            folder.SetAccessControl(security);
        }
        //!<# } #>
    }
}
